;----------------------;
; Assembler bootloader ;
;----------------------;
global loader
extern createKernel

; External variables from the linker.ld script
extern startConstructors
extern endConstructors


MODULEALIGN     equ 1 << 0
MEMINFO         equ 1 << 1
FLAGS           equ MODULEALIGN | MEMINFO
MAGIC           equ 0x1BADB002
CHECKSUM        equ -(MAGIC + FLAGS)


section .text
align 4
MultiBootHeader:
        dd MAGIC
        dd FLAGS
        dd CHECKSUM

; Create kernel stack
STACKSIZE equ 0x4000

; The loader
loader:
        mov esp, stack + STACKSIZE
        push eax        ; Magic number
        push ebx        ; Additional information

    	mov ebx, startConstructors
        jmp loopConstructors

; Calls a constructor
callConstructor:
        call [ebx]
        add ebx,4

; Loops through the constructors
loopConstructors:
        cmp ebx, endConstructors
        jb callConstructor

        cli
        call createKernel

; Hang the system after kernel execution
hang:
        jmp hang



section .bss
align 4

stack:
        resb STACKSIZE
