/**
 * Structure of a GDT entry
 */
struct GdtEntry {
        unsigned short limitLow;
        unsigned short baseLow;
        unsigned char baseMiddle;
        unsigned char access;
        unsigned char granularity;
        unsigned char baseHigh;
}__attribute__((packed));

/**
 * Structure of the GDT pointer
 */
struct GdtPointer {
        unsigned short limit;
        unsigned int base;
}__attribute__((packed));

/**
 * GdtMounter's method for mounting the global descriptor table
 */
extern "C" void _mountGdt();

/**
 * The GDT consisting of 3 GDT entries, and the GDT pointer
 */
struct GdtEntry gdt[3];
struct GdtPointer gdtPointer;

/**
 * Creates a GDT gate
 */
void createGdtGate(int index, unsigned int base, unsigned int limit,
                unsigned char access, unsigned char granularity) {
        gdt[index].baseLow = (base & 0xFFFF);
        gdt[index].baseMiddle = (base >> 16) & 0xFF;
        gdt[index].baseHigh = (base >> 24) & 0xFF;

        gdt[index].limitLow = (limit & 0xFFFF);
        gdt[index].granularity = ((limit >> 16) & 0x0F);

        gdt[index].granularity |= (granularity & 0xF0);
        gdt[index].access = access;
}

/**
 * Installs the global descriptor table.
 *
 * - Sets the limit of the GDT pointer to the size of 3 GDT entries
 * - Sets the base address to the gdt's address
 * - Installs 3 GDT gates
 * - Mounts the GDT
 */
void installGdt() {
        gdtPointer.limit = (sizeof(struct GdtEntry) * 3) - 1;
        gdtPointer.base = (unsigned int) &gdt;

        createGdtGate(0, 0, 0, 0, 0);
        createGdtGate(1, 0, 0xFFFFFFFF, 0x9A, 0xCF);
        createGdtGate(2, 0, 0xFFFFFFFF, 0x92, 0xCF);

        _mountGdt();
}
