/**
 * Structure for an IDT entry
 */
struct IdtEntry {
	unsigned short baseLow;
	unsigned short kernelSegment;
	unsigned char zero;
	unsigned char flags;
	unsigned short baseHigh;
}__attribute__((packed));

/**
 * Structure of the IDT pointer
 */
struct IdtPointer {
	unsigned short limit;
	unsigned int base;
}__attribute__((packed));

/**
 * IdtMounter's method for mounting the interrupt descriptor table
 */
extern "C" void _mountIdt();

/**
 * The IDT consisting of 256 IDT entries, and the IDT pointer
 */
struct IdtEntry idt[256];
struct IdtPointer idtPointer;

/**
 * Sets memory at the destination for a length to a value
 */
void memset(void *destination, char value, int length) {
	char *temp = (char *) destination;
	for (; length != 0; length--) {
		*temp++ = value;
	}
}

/**
 * Creates an IDT gate
 */
void createIdtGate(unsigned char index, unsigned int base,
		unsigned short kernelSegment, unsigned char flags) {
	idt[index].baseLow = base & 0xFFFF;
	idt[index].baseHigh = (base >> 16) & 0xFFFF;
	idt[index].kernelSegment = kernelSegment;
	idt[index].zero = 0;
	idt[index].flags = flags;
}

/**
 * Installs the interrupt descriptor table.
 *
 * - Sets the IDT pointers limit to the size of 256 IDT entries
 * - Sets the IDT pointers base to the address of the idt
 * - Fills memory 256 times the size of a IDT entry with zeros starting at the
 * address of the IDT
 */
void installIdt() {
	idtPointer.limit = (sizeof(struct IdtEntry) * 256) - 1;
	idtPointer.base = reinterpret_cast<unsigned int>(&idt);

	memset(&idt, 0, sizeof(struct IdtEntry) * 256);

	_mountIdt();
}
